/**********************************************************************
* Simple application for converting U-Boot environment settings 
* on the pcDuino to a editable text format.
*
* Enjoy!
* Bill
***********************************************************************
* Copyright (c) 2013 by William Greathouse, Brecksville, OH, USA
*
* Permission is granted to use the source code within this
* file in whole or in part for any use, personal or commercial,
* without restriction or limitation.
*
* No warranties, either explicit or implied, are made as to the
* suitability of this code for any purpose. Use at your own risk.
**********************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

#include "crc32.h"

static struct envfile
{
    uint32_t crc32;
    char env[];
} *envbuf;

static int verbose = 1;
static char *envsource = "/dev/nandb";

static void syntax(void)
{
    fprintf(stderr, "Syntax: envdump [filename]\n\n");
    fprintf(stderr, "One file may be specified on the command line\n"
                    "If no files are specified, the environment is\n"
                    "read from %s by default\n", envsource);
}

int main(int argc, char *argv[])
{
    FILE *envf;
    int  nr;
    char *ptr;
    uint32_t crc32check;
    long envsize = 128*1024;  // default environment size

    fprintf(stderr, "pcDuino U-Boot Environment Dump V1.0\n");
    if (argc > 2)
    {
	syntax();
	return 1;
    }

    if (argc == 2)
    {
        struct stat statbuf;

        if (stat(argv[1], &statbuf))
        {
            perror(argv[1]);
            syntax();
            return 2;
        }

        envsize = statbuf.st_size;
        envsource = argv[1];
    }

    if (verbose)
        fprintf(stderr, "%s: %ld bytes\n", envsource, envsize);

    envbuf = malloc(envsize);
    if (envbuf == NULL)
    {
        perror("Out of memory\n");
        return 3;
    }

    envf = fopen(envsource, "rb");
    if (envf == NULL)
    {
        perror(envsource);
        return 4;
    }

    nr = fread(envbuf, 1, envsize, envf);
    if (nr != envsize)
    {
        perror("Read envfile failed");
        fclose(envf);
        return 5;
    }

    if (verbose)
        fprintf(stderr, "Environment CRC32: %08x\n", envbuf->crc32);


    /* Check CRC */
    crc32check = crc32 (0, (uint8_t *)(envbuf->env), envsize-sizeof(uint32_t));

    if (crc32check != envbuf->crc32)
    {
        fprintf(stderr, "CRC FAILURE: %s CRC32 is not valid\n", argv[1]);
        fclose(envf);
        return 6;
    }

    /* Loop to extract and print environment strings */
    ptr = &envbuf->env[0];
    while (*ptr != '\0' && ptr < &envbuf->env[envsize-sizeof(uint32_t)])
    {
        while (*ptr != '\0')
        {
            putchar(*ptr);
            ptr++;
        }
        putchar('\n');
        ptr++;
    }
    
    fclose(envf);

    return 0;
}

